

var tswModalDialogMap = new Object(); //maps modal dialog id to TSWModalDialog object;

//Returns the TSWModalDialog object for an id, creating the object
//if necessary.
function tswModalDialogGetForId(id)
{
	var modalDialog = tswModalDialogMap[id];
	if(modalDialog == null)
	{
		modalDialog = new TSWModalDialog(id);
		tswModalDialogMap[id] = modalDialog;
	}
	return modalDialog;
}

function _tswModalDialogAnimate(id)
{
	tswModalDialogGetForId(id).animate();
}

//TSWModalDialog is a javascript object that represents
//the modal dialog component in the HTML document. The
//constructor takes the id of the object.
function TSWModalDialog(id)
{
	this.id = id;
	this.hasCloseButton = true;
	this.fadeInTime = 0;
	this.fadeOutTime = 0;
	
	this.overlayDiv = null;
	
	this.animationStartDate = null;
	this.animationIntervalId = null;
	this.animationFadeIn = true;
};

TSWModalDialog.prototype.getContentElement = function()
{
	return document.getElementById(this.id);
}

TSWModalDialog.prototype.showDialog = function()
{
	var content = this.getContentElement();
	if(content == null)
	{
		//no content element for the specified id
		return;
	}
	
	if(this.overlayDiv == null)
	{
		this.overlayDiv = document.createElement('div');
		this.overlayDiv.className = 'tswModalDialog';
		if(content.id != null)
		{
			this.overlayDiv.id = content.id + '_tswModalDialog';
		}
		
		if(TSWBrowserDetect.browserMatches('Explorer', null, 8))
		{
			var overlayBackground = document.createElement('div');
			overlayBackground.className = 'tswModalDialogBackground';
			this.overlayDiv.appendChild(overlayBackground);
		}
		
		var contentContainer = document.createElement('div');
		contentContainer.className = 'tswModalDialogContentContainer';
		this.overlayDiv.appendChild(contentContainer);
		
		var innerContainer = document.createElement('div');
		innerContainer.className = 'tswModalDialogContentInnerContainer';
		contentContainer.appendChild(innerContainer);
		
		innerContainer.appendChild(content);
		
		if(content.className != null && content.className.indexOf('tswDropShadow') != -1)
		{
			if(!TSWBrowserDetect.browserMatches('Explorer', null, 7))
			{
				tswDropShadowInitDiv(content);
				innerContainer.lastChild.style.width = '100%';
			}
			else
			{
				//No Drop Shadow in IE7
				content.className = content.className.replace('tswDropShadow','tswDropShadowIE');
			}
		}
		
		document.body.appendChild(this.overlayDiv);
	}
	content.style.display = 'block';
	this.overlayDiv.style.display = '';
	tswUtilsSetOpacity(this.overlayDiv, 0.0);
	
	if(this.fadeInTime > 0)
	{
		if(this.animationFadeIn && this.animationIntervalId != null)
		{
			//already fading in
			return;
		}
		
		this.animationFadeIn = true;
		this.animationStartDate = new Date;
		if(this.animationIntervalId == null)
		{
			this.animationIntervalId = setInterval("_tswModalDialogAnimate('"+this.id+"')", 25);
		}
	}
	else
	{
		tswUtilsSetOpacity(this.overlayDiv, 1.0);
	}
};

TSWModalDialog.prototype.hideDialog = function()
{
	if(this.overlayDiv == null)
	{
		return;
	}
	
	if(this.fadeOutTime > 0)
	{
		if(!this.animationFadeIn && this.animationIntervalId != null)
		{
			//already fading out
			return;
		}
		
		this.animationFadeIn = false;
		this.animationStartDate = new Date;
		if(this.animationIntervalId == null)
		{
			this.animationIntervalId = setInterval("_tswModalDialogAnimate('"+this.id+"')", 25);
		}
	}
	else
	{
		this.overlayDiv.style.display = 'none';
	}
}

TSWModalDialog.prototype.animate = function()
{
	var duration = this.animationFadeIn ? this.fadeInTime : this.fadeOutTime;
	var currentDate = new Date();
	var delta = (currentDate.getTime() - this.animationStartDate.getTime()) / duration;
	
	if(delta >= 1.0)
	{
		//complete the animation
		clearInterval(this.animationIntervalId);
		this.animationIntervalId = null;
		delta = 1.0;
		
		if(!this.animationFadeIn)
		{
			this.overlayDiv.style.display = 'none';
		}
	}
	
	var opacity = this.animationFadeIn ? delta : 1.0 - delta;
	tswUtilsSetOpacity(this.overlayDiv, opacity);
}