TSWPieChartLayout = function(options)
{
	PlotKit.Layout.call(this, 'pie', options);
}

//A chunk of code to make TSWPieChartLayout a subclass of PlotKit.Layout
function TSWPieChartLayoutSurrogateConstructor()
{
}
TSWPieChartLayoutSurrogateConstructor.prototype = PlotKit.Layout.prototype;
var TSWPieChartLayoutPrototypeObject = new TSWPieChartLayoutSurrogateConstructor();
TSWPieChartLayoutPrototypeObject.constructor = TSWPieChartLayout;
TSWPieChartLayout.prototype = TSWPieChartLayoutPrototypeObject;


TSWPieChartLayout.DataTypeNone = 'none';
TSWPieChartLayout.DataTypeValue = 'value';
TSWPieChartLayout.DataTypePercent = 'percent';


TSWPieChartLayout.prototype._evaluatePieTicks = function() {
	var isNil = MochiKit.Base.isUndefinedOrNull;
	
	var format = this.options['tswFormat'];
	if(format == null)
		format = "#";
	
	var dataType = this.options['tswDataType'];
	if(dataType == null)
		dataType = TSWPieChartLayout.DataTypePercent;
	
	if(dataType == TSWPieChartLayout.DataTypePercent)
		format += '%';
	
	var formatter = MochiKit.Format.numberFormatter(format);
	
	this.xticks = new Array();
	if (this.options.xTicks) {
		// make a lookup dict for x->slice values
		var lookup = new Array();
		for (var i = 0; i < this.slices.length; i++) {
			lookup[this.slices[i].xval] = this.slices[i];
		}
		
		for (var i =0; i < this.options.xTicks.length; i++) {
			var tick = this.options.xTicks[i];
			var slice = lookup[tick.v]; 
			var label = tick.label;
			if (slice) {
				if (isNil(label))
					label = tick.v.toString();
				
				if(dataType == TSWPieChartLayout.DataTypePercent)
					label += " (" + formatter(slice.fraction) + ")";
				else if(dataType == TSWPieChartLayout.DataTypeValue)
					label += " (" + formatter(slice.yval) + ")";
				
				this.xticks.push([tick.v, label]);
			}
		}
	}
	else {
		// we make our own labels from all the slices
		for (var i =0; i < this.slices.length; i++) {
			var slice = this.slices[i];
			var label = slice.xval;
			
			if(dataType == TSWPieChartLayout.DataTypePercent)
				label += " (" + formatter(slice.fraction) + ")";
			else if(dataType == TSWPieChartLayout.DataTypeValue)
				label += " (" + formatter(slice.yval) + ")";
			
			this.xticks.push([slice.xval, label]);
		}
	}
};

//A render that adds the capability of customizing stroke color
TSWRenderer = function(element, layout, options) {
	if(arguments.length > 0) 
		this.__init__(element, layout, options);
};
TSWRenderer.prototype = new PlotKit.SweetCanvasRenderer();
TSWRenderer.prototype.constructor = TSWRenderer;
TSWRenderer.__super__ = PlotKit.SweetCanvasRenderer.prototype;

TSWRenderer.prototype.__init__ = function(element, layout, options) {
	TSWRenderer.__super__.__init__.call(this, element, layout, options);
};

//A lot of duplicated code just to add stroke options
TSWRenderer.prototype._renderPieChart = function() {
	var context = this.element.getContext("2d");
	
	var colorCount = this.options.colorScheme.length;
	var slices = this.layout.slices;
	
	var centerx = this.area.x + this.area.w * 0.5;
	var centery = this.area.y + this.area.h * 0.5;
	var radius = Math.min(this.area.w * this.options.pieRadius, 
						  this.area.h * this.options.pieRadius);
	
	if (this.isIE) {
		centerx = parseInt(centerx);
		centery = parseInt(centery);
		radius = parseInt(radius);
	}
	
	// NOTE NOTE!! Canvas Tag draws the circle clockwise from the y = 0, x = 1
	// so we have to subtract 90 degrees to make it start at y = 1, x = 0
	
	if (!this.isIE) {
		context.save();
		var shadowColor = MochiKit.Color.Color.whiteColor().colorWithAlpha(0.2);
		context.fillStyle = shadowColor.toRGBString();
		context.shadowBlur = 5.0;
		context.shadowColor = MochiKit.Color.Color.fromHexString("#888888").toRGBString();
		context.translate(1, 1);
		context.beginPath();
		context.moveTo(centerx, centery);
		context.arc(centerx, centery, radius + 2, 0, Math.PI*2, false);
		context.closePath();
		context.fill();
		context.restore();
	}
	
	context.save();
	
	var strokeColor = this.options['tswLineColor'];
	if(strokeColor == null)
		context.strokeStyle = MochiKit.Color.Color.blackColor().toRGBString();
	else
		context.strokeStyle = strokeColor.toRGBString();
	
	var lineWidth = this.options['tswLineWidth'];
	if(lineWidth == null)
		lineWidth = 2.0;
	context.lineWidth = lineWidth;
	
	for (var i = 0; i < slices.length; i++) {
		var color = this.options.colorScheme[i%colorCount];
		context.fillStyle = color.toRGBString();
		
		var makePath = function() {
			context.beginPath();
			context.moveTo(centerx, centery);
			context.arc(centerx, centery, radius, 
						slices[i].startAngle - Math.PI/2,
						slices[i].endAngle - Math.PI/2,
						false);
			context.lineTo(centerx, centery);
			context.closePath();
		};
		
		if (Math.abs(slices[i].startAngle - slices[i].endAngle) > 0.0001) {
			if (this.options.shouldFill) {
				makePath();
				context.fill();
			}
			if (this.options.shouldStroke && lineWidth > 0) {
				makePath();
				context.stroke();
			}
		}
	}
	context.restore();
};

TSWRenderer.prototype._renderBarChart = function() {
	var bind = MochiKit.Base.bind;
	var shadowColor = MochiKit.Color.Color.blackColor().colorWithAlpha(0.1).toRGBString();
	
	var prepareFakeShadow = function(context, x, y, w, h) {
		context.fillStyle = shadowColor;
		context.fillRect(x-2, y-2, w+4, h+2); 
		context.fillStyle = shadowColor;
		context.fillRect(x-1, y-1, w+2, h+1); 
	};
	
	var colorCount = this.options.colorScheme.length;
	var colorScheme =  this.options.colorScheme;
	var setNames = PlotKit.Base.keys(this.layout.datasets);
	var setCount = setNames.length;
	
	var chooseColor = function(name) {
		for (var i = 0; i < setCount; i++) {
			if (name == setNames[i])
				return colorScheme[i%colorCount];
		}
		return colorScheme[0];
	};
	
	var drawRect = function(context, bar) {
		var x = this.area.w * bar.x + this.area.x;
		var y = this.area.h * bar.y + this.area.y;
		var w = this.area.w * bar.w;
		var h = this.area.h * bar.h;
		
		if ((w < 1) || (h < 1))
			return;		
		
		context.save();
		
		context.shadowBlur = 5.0;
		context.shadowColor = MochiKit.Color.Color.fromHexString("#888888").toRGBString();
		
		if (this.isIE) {
			context.save();
			context.fillStyle = "#cccccc";
			context.fillRect(x-2, y-2, w+4, h+2); 
			context.restore();
		}
		else {
			prepareFakeShadow(context, x, y, w, h);
		}
		
		if (this.options.shouldFill) {
			context.fillStyle = chooseColor(bar.name).toRGBString();
			context.fillRect(x, y, w, h);
		}
		
		context.shadowBlur = 0;
		
		var strokeColor = this.options['tswLineColor'];
		if(strokeColor == null)
			context.strokeStyle = MochiKit.Color.Color.blackColor().toRGBString();
		else
			context.strokeStyle = strokeColor.toRGBString();
		
		var lineWidth = this.options['tswLineWidth'];
		if(lineWidth == null)
			lineWidth = 2.0;
		context.lineWidth = lineWidth;
		
		if (this.options.shouldStroke && lineWidth > 0.0) {
			context.strokeRect(x, y, w, h);				
		}
		
		context.restore();
		
	};
	this._renderBarChartWrap(this.layout.bars, bind(drawRect, this));
};

TSWRenderer.prototype._renderLineChart = function() {
	var context = this.element.getContext("2d");
	var colorCount = this.options.colorScheme.length;
	var colorScheme = this.options.colorScheme;
	var setNames = PlotKit.Base.keys(this.layout.datasets);
	var setCount = setNames.length;
	var bind = MochiKit.Base.bind;
	
	
	for (var i = 0; i < setCount; i++) {
		var setName = setNames[i];
		var color = colorScheme[i%colorCount];
		var strokeX = this.options.strokeColorTransform;
		
		if(typeof (setName)=="string"&&(setName.length==0||setName.charAt(0)=="$")) continue;
		
		// setup graphics context
		context.save();
		
		// create paths
		var makeFillPath = function(ctx) {
			var setPoints = new Array;
			for(var i=0; i<this.layout.points.length; i++)
			{
				if(this.layout.points[i].name == setName)
				{
					setPoints.push(this.layout.points[i]);
				}
			}
			
			ctx.beginPath();
			ctx.moveTo(this.area.x, this.area.y + this.area.h);
			var addPoint = function(ctx_, point) {
				if (point.name == setName)
					ctx_.lineTo(this.area.w * point.x + this.area.x,
								this.area.h * point.y + this.area.y);
			};
			MochiKit.Iter.forEach(this.layout.points, MochiKit.Base.partial(addPoint, ctx), this);
			ctx.lineTo(this.area.w * setPoints[setPoints.length - 1].x + this.area.x,
					   this.area.h + this.area.y);
			ctx.lineTo(this.area.x, this.area.y + this.area.h);
			ctx.closePath();
		};
		var makeLinePath = function(ctx) {
			var setPoints = new Array;
			for(var i=0; i<this.layout.points.length; i++)
			{
				if(this.layout.points[i].name == setName)
				{
					setPoints.push(this.layout.points[i]);
				}
			}
			
			ctx.beginPath();
			ctx.moveTo(this.area.w * setPoints[0].x + this.area.x,
					   this.area.h * setPoints[0].y + this.area.y);
			var addPoint = function(ctx_, point) {
				if (point.name == setName)
					ctx_.lineTo(this.area.w * point.x + this.area.x,
								this.area.h * point.y + this.area.y);
			};
			
			MochiKit.Iter.forEach(setPoints, MochiKit.Base.partial(addPoint, ctx), this);
		};
		
		// faux shadow for firefox
		if (this.options.shouldFill) {
			context.save();
			if (this.isIE) {
				context.fillStyle = "#cccccc";
			}
			else {
				context.fillStyle = MochiKit.Color.Color.blackColor().colorWithAlpha(0.2).toRGBString();
			}
			context.translate(-1, -2);
			bind(makeFillPath, this)(context);
			if (this.options.shouldFill) {
				context.fill();
			}
			context.restore();
		}
		
		context.shadowBlur = 5.0;
		context.shadowColor = MochiKit.Color.Color.fromHexString("#888888").toRGBString();
		context.fillStyle = color.toRGBString();
		
		var strokeColor = this.options['tswLineColor'];
		if(!this.options.shouldFill)
			//Use the fill color to stroke if shouldFill option is false
			context.strokeStyle = color.toRGBString();
		else if(strokeColor == null)
			context.strokeStyle = MochiKit.Color.Color.blackColor().toRGBString();
		else
			context.strokeStyle = strokeColor.toRGBString();
		
		var lineWidth = this.options['tswLineWidth'];
		if(lineWidth == null)
			lineWidth = 2.0;
		context.lineWidth = lineWidth;
		
		if (this.options.shouldFill) {
			bind(makeFillPath, this)(context);
			context.fill();
		}
		if (this.options.shouldStroke) {
			bind(makeLinePath, this)(context);
			context.stroke();
		}
		context.restore();
	}
};
